import 'dart:async';

import 'package:flutter/material.dart';

class AfterRoutingHandler {
  State pageState;
  bool _animationEnd = false;
  Map<Function, dynamic> _funcDataMap = {};

  bool get _unmounted => !pageState.mounted;

  AfterRoutingHandler({
    @required this.pageState,
    @required Duration duration,
  }) {
    Timer(Duration(milliseconds: duration.inMilliseconds + 50), () {
      if (_unmounted) return;
      if (_funcDataMap.isEmpty) {
        _animationEnd = true;
      } else {
        _funcDataMap.forEach((func, data) => func(data));
      }
    });
  }

  apiUpdate<DataType>({
    @required bool fetchData,
    @required Future<DataType> apiFuture,
    @required void Function(Error) apiErrorCallback,
    @required void Function(DataType) updateDataDelegate,
  }) {
    if (!fetchData) return;
    apiFuture.then((data) {
      if (data != null) {
        _apiEnd(updateDataDelegate, data);
      } else {
        _apiEnd(apiErrorCallback, StateError("Api fail."));
      }
    }).catchError(
      (e) => () async {
            return null;
          },
    );
  }

  _apiEnd(Function callback, dynamic data) {
    print("api end isMount: $_unmounted");
    if (_unmounted) return;
    if (_animationEnd) {
      print("api end animation end.");
      callback(data);
    } else {
      print("api end animation not end.");
      _funcDataMap[callback] = data;
    }
  }
}
